# display.tcl --
#
#   This is a secondary file in the tpad library and implements the
#   Tcl code for defining the default interface of the tpad
#   application.
#
# Copyright (C) 2005  Antonio Bonifati
#
# See the file "license.terms" for information on usage and
# redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

package provide tpad 1.3

namespace eval ::tpad {
  namespace export display
}

# tpad::display --
#
#   Display a Notepad clone application in a window and start it.
#   If no additional argument is present, the editor will operate on a
#   file (as a standalone and fully independant application),
#   otherwise the additional argument must be the name of a variable.
#   If the user confirmed the editing, this variable will be set upon
#   exiting with a copy of the edited text, otherwise it will be
#   leaved unmodified. The variable initial value will be used to
#   initialize the text to be edited. Use trace(n) to execute a
#   callback every time $stringName is modified.
#
# Arguments:
#   pn      toplevel edit window.
#   stringName  (optional) name of a string variable to edit. Defaults
#           to an empty string, which means to configure the editor to
#           operate on a file instead of a variable.
# Results:
#   Returns an empty string.

proc ::tpad::display {pn {stringName {}}} {
  foreach v {p argv0 argv edit line} {
    variable $pn\::$v
  }

  if {$stringName!={}} {
    variable $pn\::var $stringName
  } else {
    variable $pn\::var {}
  }

  tk_setPalette $::tpad::palette

  # sets window manager options
  wm title $pn [format $::tpad::titlefmt [file tail $argv0] \
    [mc Untitled]]
  wm iconname $pn [file tail $argv0]
  wm command $pn [concat $argv0 $argv]
  wm group $pn $pn
  # catch the kill of the window manager
  wm protocol $pn WM_DELETE_WINDOW [namespace code "quit $pn"]
  # end of window manager options

  # builds the menus
  set m $p.menubar
  menu $m
  AmpMenu $m add cascade -menu $m.file -lab [mc &File]
  AmpMenu $m add cascade -menu $m.edit -lab [mc &Edit]
  AmpMenu $m add cascade -menu $m.format -lab [mc F&ormat]
  AmpMenu $m add cascade -menu $m.view -lab [mc &View]
  AmpMenu $m add cascade -menu $m.help -lab [mc &Help]

  set w $m.file
  menu $w -tearoff $::tpad::tearoff
  if {$var!={}} {
    AmpMenu $w add comm -lab [mc &Reset] \
      -comm [namespace code "new $pn"]
  } elseif {$edit} {
    AmpMenu $w add comm -lab [mc &New] -accel Ctrl+N \
      -comm [namespace code "new $pn"]
  } else {
    AmpMenu $w add comm -lab [mc &Clear] -accel Ctrl+N \
      -comm [namespace code "new $pn"]
  }
  AmpMenu $w add comm -lab [mc Ne&xt] \
    -comm [namespace code "next $pn"]
  AmpMenu $w add comm -lab [mc &Open...] -accel Ctrl+F12 \
    -comm [namespace code "tpadOpen $pn"]

  if {$::tpad::maxopenrec} {
    AmpMenu $w add cascade -lab [mc "Op&en Recent"] -menu $w.recent
    menu $w.recent -tearoff $::tpad::tearoff
    set entries [LoadOpenRecent $pn]
    $w.recent add separator
    $w.recent add comm -lab [mc "empty this list"] \
      -comm [namespace code "emptyOpenRecent $pn"]
    if {$entries==0} {
      $w.recent entryconf end -state disabled
    }
  }

  if {$edit} {
    if {$var=={}} {  ;# it is safe to remove this test if you want to
      AmpMenu $w add comm -lab [mc &Save] -accel Shift+F12 \
        -comm [namespace code "save $pn"]
    }
    AmpMenu $w add comm -lab [mc "Sa&ve as..."] \
      -comm [namespace code "saveAs $pn"]
  }
  $w add separator
  AmpMenu $w add comm -lab [mc "Pr&int setup..."] \
    -comm [namespace code "printSetup $pn"]
  AmpMenu $w add comm -lab [mc &Print] -accel Ctrl+Shift+F12 \
    -comm [namespace code "print $pn"]
  $w add separator
  AmpMenu $w add comm -lab [mc &Quit] \
    -comm [namespace code "quit $pn"]

  set w $m.edit
  menu $w -tearoff $::tpad::tearoff \
    -postcommand [namespace code "SetStateFromClipboard $pn $w"]
  if {$edit} {
    if {$::tpad::maxundo!=1} {
      AmpMenu $w add comm -lab [mc &Undo] -accel Ctrl+Z \
        -comm [namespace code "undo $pn"]
      AmpMenu $w add comm -lab [mc &Redo] -accel Ctrl+R \
        -comm [namespace code "redo $pn"]
    } else {
      AmpMenu $w add comm -lab [mc &Undo] -accel Ctrl+Z \
        -comm [namespace code "undoRedo $pn"]
    }
    $w add separator
    AmpMenu $w add comm -lab [mc &Cut] -accel Ctrl+X \
      -comm [namespace code "cut $pn"]
  }
  AmpMenu $w add comm -lab [mc C&opy] -accel Ctrl+C \
    -comm [namespace code "copy $pn"]
  if {$edit} {
    AmpMenu $w add comm -lab [mc &Paste] -accel Ctrl+V \
      -comm [namespace code "paste $pn"]
    AmpMenu $w add comm -lab [mc &Delete] -accel Del \
      -comm [namespace code "tpadDelete $pn"]
  }
  $w add separator
  AmpMenu $w add comm -lab [mc &Find...] -accel Ctrl+F \
    -comm [namespace code "findReplace $pn"]
  AmpMenu $w add comm -lab [mc "Find &next"] -accel F3 \
    -comm [namespace code "findNext $pn"]
  if {$edit} {
    AmpMenu $w add comm -lab [mc R&eplace...] -accel Ctrl+H \
      -comm [namespace code "findReplace $pn yes"]
  }
  AmpMenu $w add comm -lab [mc "&Goto line..."] -accel Ctrl+G \
    -comm [namespace code "goTo $pn"]
  $w add separator
  AmpMenu $w add comm -lab [mc "&Select all"] \
    -accel Ctrl+5(kp) -comm [namespace code "selectAll $pn"]
  if {$edit} {
    AmpMenu $w add comm -lab [mc Date/&time] -accel F5 \
      -comm [namespace code "insertDate $pn"]
    AmpMenu $w add comm -lab [mc "&Insert file..."] \
      -comm [namespace code "insertFile $pn"]
  }

  set w $m.format
  menu $w -tearoff $::tpad::tearoff
  AmpMenu $w add checkbutton -var ::tpad::$pn\::wrap \
    -onv word -offv none -lab [mc "&Word wrap"] \
    -comm [namespace code "toggleWordWrap $pn"]
  if {$::tpad::xfontsel!={}} {
    AmpMenu $w add comm -lab [mc "&Font..."] \
      -comm [namespace code "tpadFont $pn"]
  }
  AmpMenu $w add comm -lab [mc "&I/O Encodings"] \
    -comm [namespace code "ioEncodings $pn"]

  set w $m.view
  menu $w -tearoff $::tpad::tearoff
  AmpMenu $w add checkbutton -var ::tpad::$pn\::tool \
    -onv yes -offv no -lab [mc "&Tool bar"] \
    -comm [namespace code "toggleToolBar $pn"]
  AmpMenu $w add checkbutton -var ::tpad::$pn\::status \
    -onv yes -offv no -lab [mc "&Status bar"] \
    -comm [namespace code "toggleStatusBar $pn"]
  $w add separator
  AmpMenu $w add checkbutton -var ::tpad::$pn\::ascii \
    -onv yes -offv no -lab [mc "&Ascii table"] \
    -comm [namespace code "toggleAsciiTable $pn"]
  if {$::tpad::tidy!={}} {
    AmpMenu $w add checkbutton -var ::tpad::$pn\::ht \
      -onv yes -offv no -lab [mc "&HTML Tidy"] \
      -comm [namespace code "toggleHtmlTidy $pn"]
  }
  if {$edit && $::tpad::fmtkind!={none}} {
    AmpMenu $w add checkbutton -var ::tpad::$pn\::format \
      -onv yes -offv no -lab [mc "Text &formatter"] \
      -comm [namespace code "toggleTextFormatter $pn"]
  }
  if {$::tpad::xmlwf!={}} {
    AmpMenu $w add checkbutton -var ::tpad::$pn\::wf \
      -onv yes -offv no -lab [mc "&XML wf checker"] \
      -comm [namespace code "toggleXmlwf $pn"]
  }

  set w $m.help
  menu $w -tearoff $::tpad::tearoff
  AmpMenu $w add comm -lab [mc &Help...] -comm [namespace code help]
  AmpMenu $w add comm -lab [mc &Tips...] \
    -comm [namespace code "tip $pn"]
  AmpMenu $w add comm -lab [mc &About...] \
    -comm [namespace code "about $pn"]

  set w $p.rightmenu
  menu $w -tearoff $::tpad::tearoff \
    -postcommand [namespace code "SetStateFromClipboard $pn $w"]
  if {$edit} {
    if {$::tpad::maxundo!=1} {
      AmpMenu $w add comm -lab [mc &Undo] \
        -comm [namespace code "undo $pn"]
      AmpMenu $w add comm -lab [mc &Redo] \
        -comm [namespace code "redo $pn"]
    } else {
      AmpMenu $w add comm -lab [mc &Undo] \
        -comm [namespace code "undoRedo $pn"]
    }
    $w add separator
    AmpMenu $w add comm -lab [mc &Cut] -comm [namespace code "cut $pn"]
  }
  AmpMenu $w add comm -lab [mc C&opy] -comm [namespace code "copy $pn"]
  if {$edit} {
    AmpMenu $w add comm -lab [mc &Paste] \
      -comm [namespace code "paste $pn"]
    AmpMenu $w add comm -lab [mc &Delete] \
      -comm [namespace code "tpadDelete $pn"]
  }
  $w add separator
  AmpMenu $w add comm -lab [mc "&Select all"] \
    -comm [namespace code "selectAll $pn"]
  # end of menus' definition

  # define the widgets
  set w $p.toolBar
  frame $w -bd 2 -relief raised -padx 5 -pady 3
  if {$edit} {
    ButtonRow $pn $w.file {new open save} {new tpadOpen save}
    ButtonRow $pn $w.print {print preview} {print}
    ButtonRow $pn $w.find {find} {findNext}
    if {$::tpad::maxundo!=1} {
      ButtonRow $pn $w.clip {cut copy paste undo redo} \
        {cut copy paste undo redo}
    } else {
      ButtonRow $pn $w.clip {cut copy paste undo} \
        {cut copy paste undoRedo}
    }
    ButtonRow $pn $w.datetime {datetime} {insertDate}
    pack $w.file $w.print $w.find $w.clip $w.datetime -side left \
      -padx 5
  } else {
    ButtonRow $pn $w.file {new open} {new tpadOpen}
    ButtonRow $pn $w.print {print preview} {print}
    ButtonRow $pn $w.find {find} {findNext}
    ButtonRow $pn $w.clip {copy} {copy}
    pack $w.file $w.print $w.find $w.clip -side left -padx 5
  }

  set w $p.text
  text $w -font $::tpad::editfont -fg $::tpad::fg -bg $::tpad::bg \
    -wrap none -setgrid yes -undo $edit -maxundo $::tpad::maxundo \
    -insertofftime $::tpad::insertofftime \
    -insertontime $::tpad::insertontime \
    -yscroll [namespace code "UpdateOffset $pn"] \
    -xscroll [namespace code "$p.hscroll set"]
  $w tag configure sel -foreground $::tpad::selfg \
    -background $::tpad::selbg
  $w tag configure dupsel -foreground $::tpad::selfg \
    -background $::tpad::selbg
  scrollbar $p.vscroll -comm [namespace code "$w yview"] \
    -width $::tpad::scrollwidth
  scrollbar $p.hscroll -comm [namespace code "$w xview"] \
    -orient horizontal -width $::tpad::scrollwidth

  set w $p.statusBar
  frame $w -pady 2
  label $w.filename -anchor w -bd 1 -font $::tpad::statusfont \
    -text [mc Untitled]
  label $w.modified -relief sunken -width [mcmax Modified Readonly] \
    -bd 1 -font $::tpad::statusfont
  label $w.lines -relief sunken -text 1 \
    -width 6 -bd 1 -font $::tpad::statusfont
  label $w.fraction -relief sunken -width 8 -bd 1 \
    -font $::tpad::statusfont
  label $w.coords -relief sunken -text 1,1 \
    -width 8 -bd 1 -font $::tpad::statusfont
  pack $w.filename -side left -exp yes -fill x -padx 2
  pack $w.modified $w.lines $w.fraction $w.coords -side left -padx 2
  # end of widgets' definition

  # install or disable X event handlers
  if {$var=={}} {
    bind $pn <Control-Key-n> [namespace code "new $pn"]
    # this should be preferable than a "bind Text <Control-Key-n> {}"
    bind $p.text <Control-Key-n> [namespace code "new $pn; break"]
  }
  bind $pn <Control-Key-F12> [namespace code "tpadOpen $pn"]
  bind $pn <Control-Shift-Key-F12> [namespace code "print $pn"]
  catch {bind $pn <Control-Shift-Key-XF86_Switch_VT_12> [
    namespace code "print $pn"
  ]}
  catch {bind $p.text <Control-Key-KP_Begin> \
    [namespace code "selectAll $pn"]}
  bind $p.text <Control-Key-Clear> [namespace code "selectAll $pn"]
  bind $p.text <Control-Key-f> \
    [namespace code "findReplace $pn; break"]
  bind $p.text <Control-Key-h> \
    [namespace code "findReplace $pn yes; break"]
  bind $p.text <Control-Key-g> [namespace code "goTo $pn"]
  bind $p.text <Key-F3> [namespace code "findNext $pn"]
  bind $p.text <<Modified>> [namespace code "Modified $pn"]
  bind $p.text <KeyRelease> [namespace code "
    updateStatusBar $pn
    SetStateFromSelection $pn
  "]
  bind $p.text <ButtonRelease> [namespace code "updateStatusBar $pn"]
  bind $p.text <KeyPress> [namespace code "updateStatusBar $pn"]
  bind $p.text <ButtonPress> [namespace code "updateStatusBar $pn"]
  bind $p.text <ButtonPress-3> \
    [namespace code "tk_popup $p.rightmenu %X %Y"]
  bind $p.text <<Selection>> [namespace code "
    DuplicateSelection $pn
    SetStateFromSelection $pn
  "]
  if {$edit} {
    if {$var=={}} {
      bind $pn <Shift-Key-F12> [namespace code "save $pn"]
      catch {bind $pn <Shift-Key-XF86_Switch_VT_12> \
        [namespace code "save $pn"]}
    }
    bind $p.text <Key-F5> [namespace code "insertDate $pn"]

    # here it seems better to use break than to destroy the <<Undo>>
    # binding for all Text widgets using "bind Text <<Undo>> {}"
    bind $p.text <<Redo>> [namespace code "redo $pn; break"]
    if {$::tpad::maxundo!=1} {
      event add <<Redo>> <Control-Key-r>
      bind $p.text <<Undo>> [namespace code "undo $pn; break"]
    } else {
      bind $p.text <<Undo>> [namespace code "undoRedo $pn; break"]
    }
    InstallInterceptor $pn

    bind $p.text <<Paste>> [namespace code "paste $pn; break"]
  }
  # end of handlers' definition

  # packing
  $pn config -menu $m
  grid $p.toolBar -row 0 -column 0 -columnspan 2 -sticky ew
  grid $p.text -row 1 -column 0 -sticky news
  grid $p.vscroll -row 1 -column 1 -sticky ns
  grid $p.hscroll -row 2 -column 0 -sticky ew
  grid $p.statusBar -row 3 -column 0 -columnspan 2 -sticky ew
  grid rowconfigure $pn 1 -weight 1
  grid columnconfigure $pn 0 -weight 1
  focus $p.text
  # end of packing

  toggleToolBar $pn
  toggleWordWrap $pn
  toggleStatusBar $pn
  toggleAsciiTable $pn
  toggleTextFormatter $pn
  toggleXmlwf $pn
  toggleHtmlTidy $pn
  SetStateFromClipboard $pn $p.menubar.edit
  SetStateFromClipboard $pn $p.rightmenu

  # this is phase 2 of command line parameters handling i.e.
  # parse remaining arguments (or multiple file sequential editing)
  if { ![next $pn] } {
    new $pn
  }
  if {$line!={}} {
    goToLine $pn $line
  }

  # make sure to display everything first so the events generated
  # below can't be ignored and the optional modal tip windows
  # always pops up over a complete interface
  update idletasks

  event generate $p.text <<Selection>>
  event generate $p.text <<Modified>>

  if {$::tpad::tip} {tip $pn}

  return
}
