# tpad.tcl --
#
#   This is the primary file in the tpad library and implements
#   the Tcl code for both exported and internal commands.
#
# Copyright (C) 2005  Antonio Bonifati
#
# See the file "license.terms" for information on usage and
# redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

...

package provide tpad 1.3

namespace eval ::tpad {
  # Init commands: note that the tpad command must be called
  # *before* any other exported command and the default
  # version of display is defined in display.tcl.
  namespace export tpad

  # Public editor menu commands.
  namespace export new next tpadOpen save saveAs printSetup print quit \
    undoRedo undo redo cut copy paste tpadDelete findReplace findNext \
    goTo selectAll insertDate insertFile toggleToolBar toggleWordWrap \
    tpadFont ioEncodings toggleStatusBar toggleAsciiTable toggleHtmlTidy \
    toggleTextFormatter toggleXmlwf help tip about

  # Public useful and safe commands (not menu commands).
  namespace export lineNo lines goToLine updateStatusBar whatIs cmdHelp \
    emptyOpenRecent openRecentLength getInputEnc getOutputEnc fortune
}

## Private commands.

...

## Public useful and safe commands (not menu commands).

# tpad::getInputEnc --
#
#   Tell what encoding tpad is currently used when reading files.
#   This is one of the elements of the list returned by the
#   [encoding names] command.
#
# Arguments:
#   pn  toplevel edit window.
# Results:
#   Returns the current input encoding.

...

# tpad::getOutputEnc --
#
#   Similar to getInputEnc, but the encoding
#   is that used when writing files.
#
# Arguments:
#   pn  toplevel edit window.
# Results:
#   Returns the current output encoding.

...

# tpad::fortune --
#
#   Return a random, hopefully useful, tip.
#
# Returns:
#   Returns a tip about using tpad as a multiline string.

...

# tpad::lineNo --
#
#   Given an index in the text, returns the corresponding line number.
#   Lines are numbered from 1 for consistency with other UNIX programs
#   that use this numbering scheme.
#
# Arguments:
#   pn  toplevel edit window.
#   index   index in the text.
# Results:
#   Returns the line number corresponding to index.

...

# tpad::lines --
#
#   Find out how many lines the text currently has. An empty text has 1
#   line, not 0, because there is always a newline as the last character.
#
# Arguments:
#   pn  toplevel edit window.
# Results:
#   Returns the total number of lines in the text.

...

# tpad::goToLine --
#
#   If row is a valid line number in the text, set the insertion
#   cursor to the specified row:col position and adjust the view.
#
# Arguments:
#   pn  toplevel edit window.
#   row row number. From 1.
#   col (optional) column number. From 0. Defaults to 0,
#       which means the first character within a line.
# Returns:
#   Returns a boolean yes if the cursor positioning was successful,
#   otherwise pops up an error message window and returns no.

...

# tpad::updateStatusBar --
#
#   Update the status bar. Invoked every time a key or the left mouse
#   button is pressed or released, after a successful search, a cursor
#   repositioning, a paste operation and so on.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::whatIs --
#
#   Return program brief description with version number.
#
# Returns:
#   Returns a localized string describing tpad.

...

# tpad::cmdHelp --
#
#   Return program brief description as tpad::whatIs
#   plus command line syntax.
#
# Returns:
#   Returns a localized string describing tpad
#   and how to use it from the command line.

...

## End of public useful commands (not menu commands).


## Public editor menu commands.

# tpad::new --
#
#   Switch to edit a new empty text.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::next --
#
#   Arranges for editing the next file in sequence if there's one,
#   otherwise simply disables the Next option.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns a boolean yes if a next file in sequence is
#   actually loaded or created as new, no otherwise.

...

# tpad::tpadOpen --
#
#   Open a new file for editing.
#
# Arguments:
#   pn      toplevel edit window.
#   newFilename (optional) name of the file to edit. Defaults to an
#           empty string, which means to ask the user.
# Returns:
#   Returns an empty string.

...

# tpad::save --
#
#   Write the file back. Ask the user for the name of the file is not
#   defined yet. Also ask for confirmation if the file is read-only.
#   or appears to be more recently than this copy.
#
#   This proc no more does nothing if the text hasn't been modified from
#   last saved version, because one may want to save the text to convert
#   it to another encoding, without modifying it and also because these
#   optimizations are automatic in modern OS's.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns a boolean yes if the file was saved, no if the user has
#   cancelled the operation.

...

# tpad::saveAs --
#
#   Save the current copy of the text on a file with a new name.
#   This will be the new file to be edited.
#
# Arguments:
#   pn      toplevel edit window.
#   newFilename (optional) new name of the file. Defaults to
#           an empty string, which means to ask the user.
# Returns:
#   Returns a boolean yes if the user has chosen to save,
#   no if the user has cancelled the operation.

...

# tpad::tpadFont --
#
#   Display a dialog allowing the user to change
#   the editor and ascii table plugin font.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::ioEncodings --
#
#   Display a modal window allowing the user to set the encodings used
#   to read/write files. tpad will convert from UTF-8 (used internally
#   by Tcl) to the specified encoding whenever reads or writes files.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::printSetup --
#
#   Open the modal Print setup dialog box.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::print --
#
#   Print the text edited so far, piping it into lpr(1) or similar.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::quit --
#
#   Quit from from the application, if the user confirms.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::undoRedo --
#
#   Implement one level undo/redo, i.e. toggle
#   between undo and redo whichever is applicable.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::undo --
#
#   Undo the last edit action. Do nothing when the undo stack is empty.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::redo --
#
#   Reapply the last undone edits provided no other edits were done
#   since then. Do nothing when the redo stack is empty.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::cut --
#
#   Copy the selection in the widget to the clipboard and delete the
#   selection. If there is no selection in the widget do nothing.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::copy --
#
#   Copy the selection in the widget to the clipboard,
#   if there is a selection.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::paste --
#
#   Inserts the contents of the clipboard at the position of the
#   insertion cursor. Pasting over a selection replaces it.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::tpadDelete --
#
#   Delete the selection, if there is one in the text.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::findReplace --
#
#   Opens the find or find&replace dialog box if it isn't already opened.
#
# Arguments:
#   pn  toplevel edit window.
#   replace (optional) boolean. If it is yes the replace features will be
#       included. Defaults to no, which means allow only searches,
#       not string replacement.
# Returns:
#   Returns an empty string.

...

# tpad::findNext --
#
#   Find the next occurrence of the pattern. If the pattern is empty
#   (not defined) or the Find window is opened, invoke findReplace instead.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Same as FindPattern if the search had place,
#   otherwise returns an empty string.

...

# tpad::goTo --
#
#   Opens the modal Goto line dialog box.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::selectAll --
#
#   Select the entire text.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::insertDate --
#
#   Insert locale specific date and time just before the insertion cursor.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::insertFile --
#
#   Insert a file just below the cursor.
#
# Arguments:
#   pn      toplevel edit window.
#   insFilename (optional) name of the file to insert. Defaults to an
#           empty string, which means to ask the user.
# Returns:
#   Returns an empty string.

...

# tpad::toggleToolBar --
#
#   Display/hide the toolbar.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::toggleWordWrap --
#
#   Enable/disable word wrapping mode.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::toggleAsciiTable --
#
#   Display/hide the Ascii Table plugin.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::toggleHtmlTidy --
#
#   Display/hide the HTML Tidy plugin.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::toggleTextFormatter --
#
#   Display/hide the Text Formatter plugin.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::toggleXmlwf --
#
#   Display/hide the Xmlwf plugin.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::toggleStatusBar --
#
#   Display/hide the status bar.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::tip --
#
#   Print a random tip about using tpad.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::emptyOpenRecent --
#
#   Empty the open recent file list in memory.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

# tpad::openRecentLength --
#
#   Compute the length of the open recent file list.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns a decimal string giving the number of elements in the list.

...

# tpad::help --
#
#   Fire up the browser to read tpad documentation.
#
# Returns:
#   Returns an empty string.

...

# tpad::about --
#
#   Pop up the modal about message window.
#
# Arguments:
#   pn  toplevel edit window.
# Returns:
#   Returns an empty string.

...

## End of public editor menu commands.


## Init commands.

# tpad::tpad
#
#   Given pathname (pn), a toplevel window widget
#   created by toplevel(n) or the main window (.),
#   build up an instance of the tpad editor inside it.
#
# Arguments:
#   name    name of the executable. Must be tpad for the editor,
#       tview for the viewer.
#   args    list of options with the same syntax used on the command line.
#       Indeed it can be set to $argv or even left empty.
# Returns:
#   Returns an empty string.

...

## End of init commands.


...
